@@ -24,7 +24,218 @@ from pipenv.utils.pipfile import ensure_pipfile
 from pipenv.utils.project import ensure_project
 from pipenv.utils.requirements import add_index_to_pipfile, import_requirements
 from pipenv.utils.shell import temp_environ
-from pipenv.utils.virtualenv import cleanup_virtualenv, do_create_virtualenv
+
+
+def handle_new_packages(
+    project,
+    packages,
+    editable_packages,
+    dev,
+    pre,
+    system,
+    pypi_mirror,
+    extra_pip_args,
+    pipfile_categories,
+    perform_upgrades=True,
+    index=None,
+):
+    from pipenv.routines.update import do_update
+
+    new_packages = []
+    if packages or editable_packages:
+
+        pkg_list = packages + [f"-e {pkg}" for pkg in editable_packages]
+
+        for pkg_line in pkg_list:
+            console.print(f"Installing {pkg_line}...", style="bold green")
+            with temp_environ():
+                if not system:
+                    os.environ["PIP_USER"] = "0"
+                    if "PYTHONHOME" in os.environ:
+                        del os.environ["PYTHONHOME"]
+
+                try:
+                    pkg_requirement, _ = expansive_install_req_from_line(
+                        pkg_line, expand_env=True
+                    )
+                    if index:
+                        source = project.get_index_by_name(index)
+                        default_index = project.get_default_index()["name"]
+                        if not source:
+                            index_name = add_index_to_pipfile(project, index)
+                            if index_name != default_index:
+                                pkg_requirement.index = index_name
+                        elif source["name"] != default_index:
+                            pkg_requirement.index = source["name"]
+                except ValueError as e:
+                    err.print(f"[red]WARNING[/red]: {e}")
+                    err.print(
+                        environments.PIPENV_SPINNER_FAIL_TEXT.format(
+                            "Installation Failed"
+                        )
+                    )
+                    sys.exit(1)
+
+                try:
+                    if pipfile_categories:
+                        for category in pipfile_categories:
+                            added, cat, normalized_name = project.add_package_to_pipfile(
+                                pkg_requirement, pkg_line, dev, category
+                            )
+                            if added:
+                                new_packages.append((normalized_name, cat))
+                    else:
+                        added, cat, normalized_name = project.add_package_to_pipfile(
+                            pkg_requirement, pkg_line, dev
+                        )
+                        if added:
+                            new_packages.append((normalized_name, cat))
+                except ValueError:
+                    import traceback
+
+                    err.print(f"[bold][red]Error:[/red][/bold] {traceback.format_exc()}")
+                    err.print(
+                        environments.PIPENV_SPINNER_FAIL_TEXT.format(
+                            "Failed adding package to Pipfile"
+                        )
+                    )
+
+                console.print(
+                    environments.PIPENV_SPINNER_OK_TEXT.format("Installation Succeeded")
+                )
+
+        # Update project settings with pre-release preference.
+        if pre:
+            project.update_settings({"allow_prereleases": pre})
+
+    # Use the update routine for new packages
+    if perform_upgrades:
+        try:
+            do_update(
+                project,
+                dev=dev,
+                pre=pre,
+                packages=packages,
+                editable_packages=editable_packages,
+                pypi_mirror=pypi_mirror,
+                index_url=index,
+                extra_pip_args=extra_pip_args,
+                categories=pipfile_categories,
+            )
+            return new_packages, True
+        except Exception:
+            for pkg_name, category in new_packages:
+                project.remove_package_from_pipfile(pkg_name, category)
+            raise
+
+    return new_packages, False
+
+
+def handle_lockfile(
+    project,
+    packages,
+    ignore_pipfile,
+    skip_lock,
+    system,
+    allow_global,
+    deploy,
+    pre,
+    pypi_mirror,
+    categories,
+):
+    """Handle the lockfile, updating if necessary.  Returns True if package updates were applied."""
+    if (
+        (project.lockfile_exists and not ignore_pipfile)
+        and not skip_lock
+        and not packages
+    ):
+        old_hash = project.get_lockfile_hash()
+        new_hash = project.calculate_pipfile_hash()
+        if new_hash != old_hash:
+            if deploy:
+                console.print(
+                    f"Your Pipfile.lock ({old_hash}) is out of date. Expected: ({new_hash}).",
+                    style="red",
+                )
+                raise exceptions.DeployException
+            handle_outdated_lockfile(
+                project,
+                packages,
+                old_hash=old_hash,
+                new_hash=new_hash,
+                system=system,
+                allow_global=allow_global,
+                skip_lock=skip_lock,
+                pre=pre,
+                pypi_mirror=pypi_mirror,
+                categories=categories,
+            )
+    elif not project.lockfile_exists and not skip_lock:
+        handle_missing_lockfile(project, system, allow_global, pre, pypi_mirror)
+
+
+def handle_outdated_lockfile(
+    project,
+    packages,
+    old_hash,
+    new_hash,
+    system,
+    allow_global,
+    skip_lock,
+    pre,
+    pypi_mirror,
+    categories,
+):
+    """Handle an outdated lockfile returning True if package updates were applied."""
+    if (system or allow_global) and not (project.s.PIPENV_VIRTUALENV):
+        err.print(
+            f"Pipfile.lock ({old_hash}) out of date, but installation uses --system so"
+            f" re-building lockfile must happen in isolation."
+            f" Please rebuild lockfile in a virtualenv. Continuing anyway...",
+            style="yellow",
+        )
+    else:
+        if old_hash:
+            msg = (
+                "Pipfile.lock ({0}) out of date: run `pipenv lock` to update to ({1})..."
+            )
+        else:
+            msg = "Pipfile.lock is corrupt, replaced with ({1})..."
+        err.print(
+            msg.format(old_hash, new_hash),
+            style="bold yellow",
+        )
+        if not skip_lock:
+            do_lock(
+                project,
+                system=system,
+                pre=pre,
+                write=True,
+                pypi_mirror=pypi_mirror,
+                categories=None,
+            )
+
+
+def handle_missing_lockfile(project, system, allow_global, pre, pypi_mirror):
+    if (system or allow_global) and not project.s.PIPENV_VIRTUALENV:
+        raise exceptions.PipenvOptionsError(
+            "--system",
+            "--system is intended to be used for Pipfile installation, "
+            "not installation of specific packages. Aborting.\n"
+            "See also: --deploy flag.",
+        )
+    else:
+        err.print(
+            "Pipfile.lock not found, creating...",
+            style="bold",
+        )
+        do_lock(
+            project,
+            system=system,
+            pre=pre,
+            write=True,
+            pypi_mirror=pypi_mirror,
+        )
 
 
 def do_install(
@@ -42,32 +253,108 @@ def do_install(
     deploy=False,
     site_packages=None,
     extra_pip_args=None,
-    categories=None,
+    pipfile_categories=None,
     skip_lock=False,
 ):
     requirements_directory = fileutils.create_tracked_tempdir(
         suffix="-requirements", prefix="pipenv-"
     )
-    warnings.filterwarnings("default", category=ResourceWarning)
+    warnings.filterwarnings("ignore", category=ResourceWarning)
     packages = packages if packages else []
     editable_packages = editable_packages if editable_packages else []
     package_args = [p for p in packages if p] + [p for p in editable_packages if p]
-    skip_requirements = False
-    # Don't search for requirements.txt files if the user provides one
-    if requirementstxt or package_args or project.pipfile_exists:
-        skip_requirements = True
-    # Ensure that virtualenv is available and pipfile are available
+    new_packages = []
+    if dev and not pipfile_categories:
+        pipfile_categories = ["dev-packages"]
+    elif not pipfile_categories:
+        pipfile_categories = ["packages"]
+
     ensure_project(
         project,
         python=python,
         system=system,
         warn=True,
         deploy=deploy,
-        skip_requirements=skip_requirements,
+        skip_requirements=False,
         pypi_mirror=pypi_mirror,
         site_packages=site_packages,
-        categories=categories,
+        pipfile_categories=pipfile_categories,
     )
+
+    do_init(
+        project,
+        package_args,
+        system=system,
+        allow_global=system,
+        deploy=deploy,
+        pypi_mirror=pypi_mirror,
+        skip_lock=skip_lock,
+        categories=pipfile_categories,
+    )
+
+    do_install_validations(
+        project,
+        package_args,
+        requirements_directory,
+        dev=dev,
+        system=system,
+        ignore_pipfile=ignore_pipfile,
+        requirementstxt=requirementstxt,
+        pre=pre,
+        deploy=deploy,
+        categories=pipfile_categories,
+        skip_lock=skip_lock,
+    )
+    if not deploy:
+        new_packages, _ = handle_new_packages(
+            project,
+            packages,
+            editable_packages,
+            dev=dev,
+            pre=pre,
+            system=system,
+            pypi_mirror=pypi_mirror,
+            extra_pip_args=extra_pip_args,
+            pipfile_categories=pipfile_categories,
+            perform_upgrades=not skip_lock,
+            index=index,
+        )
+
+    try:
+        if dev:  # Install both develop and default package categories from Pipfile.
+            pipfile_categories = ["default", "develop"]
+        do_install_dependencies(
+            project,
+            dev=dev,
+            allow_global=system,
+            requirements_dir=requirements_directory,
+            pypi_mirror=pypi_mirror,
+            extra_pip_args=extra_pip_args,
+            categories=pipfile_categories,
+            skip_lock=skip_lock,
+        )
+    except Exception as e:
+        # If we fail to install, remove the package from the Pipfile.
+        for pkg_name, category in new_packages:
+            project.remove_package_from_pipfile(pkg_name, category)
+        raise e
+
+    sys.exit(0)
+
+
+def do_install_validations(
+    project,
+    package_args,
+    requirements_directory,
+    dev=False,
+    system=False,
+    ignore_pipfile=False,
+    requirementstxt=False,
+    pre=False,
+    deploy=False,
+    categories=None,
+    skip_lock=False,
+):
     # Don't attempt to install develop and default packages if Pipfile is missing
     if not project.pipfile_exists and not (package_args or dev):
         if not (ignore_pipfile or deploy):
@@ -151,211 +438,10 @@ def do_install(
                 err.print(str(traceback), style="yellow")
                 sys.exit(1)
 
-    # Allow more than one package to be provided.
-    package_args = list(packages) + [f"-e {pkg}" for pkg in editable_packages]
-    # Install all dependencies, if none was provided.
-    # This basically ensures that we have a pipfile and lockfile, then it locks and
-    # installs from the lockfile
-    new_packages = []
-    if not packages and not editable_packages:
-        # Update project settings with prerelease preference.
-        if pre:
-            project.update_settings({"allow_prereleases": pre})
-        do_init(
-            project,
-            dev=dev,
-            allow_global=system,
-            ignore_pipfile=ignore_pipfile,
-            system=system,
-            deploy=deploy,
-            pre=pre,
-            requirements_dir=requirements_directory,
-            pypi_mirror=pypi_mirror,
-            extra_pip_args=extra_pip_args,
-            categories=categories,
-            skip_lock=skip_lock,
-        )
-
-    # This is for if the user passed in dependencies, then we want to make sure we
-    else:
-        # make a tuple of (display_name, entry)
-        pkg_list = packages + [f"-e {pkg}" for pkg in editable_packages]
-        if not system and not project.virtualenv_exists:
-            do_init(
-                project,
-                dev=dev,
-                system=system,
-                allow_global=system,
-                requirements_dir=requirements_directory,
-                deploy=deploy,
-                pypi_mirror=pypi_mirror,
-                extra_pip_args=extra_pip_args,
-                categories=categories,
-                skip_lock=skip_lock,
-            )
-
-        for pkg_line in pkg_list:
-            console.print(
-                f"Installing {pkg_line}...",
-                style="bold green",
-            )
-            # pip install:
-            with temp_environ(), console.status(
-                "Installing...", spinner=project.s.PIPENV_SPINNER
-            ) as st:
-                if not system:
-                    os.environ["PIP_USER"] = "0"
-                    if "PYTHONHOME" in os.environ:
-                        del os.environ["PYTHONHOME"]
-                st.console.print(f"Resolving {pkg_line}...", markup=False)
-                try:
-                    pkg_requirement = expansive_install_req_from_line(
-                        pkg_line, expand_env=True
-                    )
-                except ValueError as e:
-                    err.print(f"[red]WARNING[/red]: {e}")
-                    err.print(
-                        environments.PIPENV_SPINNER_FAIL_TEXT.format(
-                            "Installation Failed"
-                        )
-                    )
-                    sys.exit(1)
-                st.update(f"Installing {pkg_requirement.name}...")
-                if categories:
-                    pipfile_sections = ""
-                    for c in categories:
-                        pipfile_sections += f"[{c}]"
-                elif dev:
-                    pipfile_sections = "[dev-packages]"
-                else:
-                    pipfile_sections = "[packages]"
-                # Add the package to the Pipfile.
-                if index:
-                    source = project.get_index_by_name(index)
-                    default_index = project.get_default_index()["name"]
-                    if not source:
-                        index_name = add_index_to_pipfile(project, index)
-                        if index_name != default_index:
-                            pkg_requirement.index = index_name
-                    elif source["name"] != default_index:
-                        pkg_requirement.index = source["name"]
-                try:
-                    if categories:
-                        for category in categories:
-                            added, cat, normalized_name = project.add_package_to_pipfile(
-                                pkg_requirement, pkg_line, dev, category
-                            )
-                            if added:
-                                new_packages.append((normalized_name, cat))
-                                st.console.print(
-                                    f"[bold]Added [green]{normalized_name}[/green][/bold] to Pipfile's "
-                                    f"[yellow]\\{pipfile_sections}[/yellow] ..."
-                                )
-                    else:
-                        added, cat, normalized_name = project.add_package_to_pipfile(
-                            pkg_requirement, pkg_line, dev
-                        )
-                        if added:
-                            new_packages.append((normalized_name, cat))
-                            st.console.print(
-                                f"[bold]Added [green]{normalized_name}[/green][/bold] to Pipfile's "
-                                f"[yellow]\\{pipfile_sections}[/yellow] ..."
-                            )
-                except ValueError:
-                    import traceback
-
-                    err.print(f"[bold][red]Error:[/red][/bold] {traceback.format_exc()}")
-                    err.print(
-                        environments.PIPENV_SPINNER_FAIL_TEXT.format(
-                            "Failed adding package to Pipfile"
-                        )
-                    )
-                # ok has a nice v in front, should do something similar with rich
-                st.console.print(
-                    environments.PIPENV_SPINNER_OK_TEXT.format("Installation Succeeded")
-                )
-            # Update project settings with pre-release preference.
-            if pre:
-                project.update_settings({"allow_prereleases": pre})
-        try:
-            do_init(
-                project,
-                dev=dev,
-                system=system,
-                allow_global=system,
-                requirements_dir=requirements_directory,
-                deploy=deploy,
-                pypi_mirror=pypi_mirror,
-                extra_pip_args=extra_pip_args,
-                categories=categories,
-                skip_lock=skip_lock,
-            )
-        except Exception as e:
-            # If we fail to install, remove the package from the Pipfile.
-            for pkg_name, category in new_packages:
-                project.remove_package_from_pipfile(pkg_name, category)
-            raise e
-    sys.exit(0)
-
-
-def do_sync(
-    project,
-    dev=False,
-    python=None,
-    bare=False,
-    user=False,
-    clear=False,
-    unused=False,
-    pypi_mirror=None,
-    system=False,
-    deploy=False,
-    extra_pip_args=None,
-    categories=None,
-    site_packages=False,
-):
-    # The lock file needs to exist because sync won't write to it.
-    if not project.lockfile_exists:
-        raise exceptions.LockfileNotFound("Pipfile.lock")
-
-    # Ensure that virtualenv is available if not system.
-    ensure_project(
-        project,
-        python=python,
-        validate=False,
-        system=system,
-        deploy=deploy,
-        pypi_mirror=pypi_mirror,
-        clear=clear,
-        site_packages=site_packages,
-    )
-
-    # Install everything.
-    requirements_dir = fileutils.create_tracked_tempdir(
-        suffix="-requirements", prefix="pipenv-"
-    )
-    if system:
-        project.s.PIPENV_USE_SYSTEM = True
-        os.environ["PIPENV_USE_SYSTEM"] = "1"
-    do_init(
-        project,
-        dev=dev,
-        allow_global=system,
-        requirements_dir=requirements_dir,
-        ignore_pipfile=True,  # Don't check if Pipfile and lock match.
-        pypi_mirror=pypi_mirror,
-        deploy=deploy,
-        system=system,
-        extra_pip_args=extra_pip_args,
-        categories=categories,
-    )
-    if not bare:
-        console.print("[green]All dependencies are now up-to-date![/green]")
-
 
 def do_install_dependencies(
     project,
     dev=False,
-    dev_only=False,
     bare=False,
     allow_global=False,
     ignore_hashes=False,
@@ -371,31 +457,30 @@ def do_install_dependencies(
     """
     procs = queue.Queue(maxsize=1)
     if not categories:
-        if dev and dev_only:
-            categories = ["dev-packages"]
-        elif dev:
+        if dev:
             categories = ["packages", "dev-packages"]
         else:
             categories = ["packages"]
 
-    for category in categories:
-        # Load the lockfile if it exists, or if dev_only is being used.
+    for pipfile_category in categories:
         lockfile = None
         pipfile = None
         if skip_lock:
             ignore_hashes = True
             if not bare:
                 console.print("Installing dependencies from Pipfile...", style="bold")
-            pipfile = project.get_pipfile_section(category)
+            pipfile = project.get_pipfile_section(pipfile_category)
         else:
             lockfile = project.get_or_create_lockfile(categories=categories)
             if not bare:
+                lockfile_category = get_lockfile_section_using_pipfile_category(
+                    pipfile_category
+                )
                 console.print(
-                    f"Installing dependencies from Pipfile.lock "
+                    f"Installing dependencies from Pipfile.lock [{lockfile_category}]"
                     f"({lockfile['_meta'].get('hash', {}).get('sha256')[-6:]})...",
                     style="bold",
                 )
-        dev = dev or dev_only
         if skip_lock:
             deps_list = []
             for req_name, pipfile_entry in pipfile.items():
@@ -409,8 +494,13 @@ def do_install_dependencies(
                     )
                 )
         else:
+            lockfile_category = get_lockfile_section_using_pipfile_category(
+                pipfile_category
+            )
             deps_list = list(
-                lockfile.get_requirements(dev=dev, only=dev_only, categories=[category])
+                lockfile.get_requirements(
+                    dev=dev, only=False, categories=[lockfile_category]
+                )
             )
         editable_or_vcs_deps = [
             (dep, pip_line) for dep, pip_line in deps_list if (dep.link and dep.editable)
@@ -432,7 +522,9 @@ def do_install_dependencies(
         if skip_lock:
             lockfile_section = pipfile
         else:
-            lockfile_category = get_lockfile_section_using_pipfile_category(category)
+            lockfile_category = get_lockfile_section_using_pipfile_category(
+                pipfile_category
+            )
             lockfile_section = lockfile[lockfile_category]
         batch_install(
             project,
@@ -582,114 +674,35 @@ def _cleanup_procs(project, procs):
 
 def do_init(
     project,
-    dev=False,
-    dev_only=False,
+    packages=None,
     allow_global=False,
     ignore_pipfile=False,
     system=False,
     deploy=False,
     pre=False,
-    requirements_dir=None,
     pypi_mirror=None,
-    extra_pip_args=None,
-    categories=None,
     skip_lock=False,
+    categories=None,
 ):
-    """Executes the init functionality."""
-    python = None
-    if project.s.PIPENV_PYTHON is not None:
-        python = project.s.PIPENV_PYTHON
-    elif project.s.PIPENV_DEFAULT_PYTHON_VERSION is not None:
-        python = project.s.PIPENV_DEFAULT_PYTHON_VERSION
-    if categories is None:
-        categories = []
-
-    if not system and not project.s.PIPENV_USE_SYSTEM:
-        if not project.virtualenv_exists:
-            try:
-                do_create_virtualenv(project, python=python, pypi_mirror=pypi_mirror)
-            except KeyboardInterrupt:
-                cleanup_virtualenv(project, bare=False)
-                sys.exit(1)
-    # Ensure the Pipfile exists.
+    """Initialize the project, ensuring that the Pipfile and Pipfile.lock are in place.
+    Returns True if packages were updated + installed.
+    """
     if not deploy:
         ensure_pipfile(project, system=system)
-    if not requirements_dir:
-        requirements_dir = fileutils.create_tracked_tempdir(
-            suffix="-requirements", prefix="pipenv-"
-        )
-    # Write out the lockfile if it doesn't exist, but not if the Pipfile is being ignored
-    if (project.lockfile_exists and not ignore_pipfile) and not skip_lock:
-        old_hash = project.get_lockfile_hash()
-        new_hash = project.calculate_pipfile_hash()
-        if new_hash != old_hash:
-            if deploy:
-                console.print(
-                    f"Your Pipfile.lock ({old_hash[-6:]}) is out of date.  Expected: ({new_hash[-6:]}).",
-                    style="red",
-                )
-                raise exceptions.DeployException
-            if (system or allow_global) and not (project.s.PIPENV_VIRTUALENV):
-                err.print(
-                    f"Pipfile.lock ({old_hash[-6:]}) out of date, but installation uses --system so"
-                    f"re-building lockfile must happen in isolation."
-                    f" Please rebuild lockfile in a virtualenv.  Continuing anyway...",
-                    style="yellow",
-                )
-            else:
-                if old_hash:
-                    msg = "Pipfile.lock ({0}) out of date, updating to ({1})..."
-                else:
-                    msg = "Pipfile.lock is corrupt, replaced with ({1})..."
-                err.print(
-                    msg.format(old_hash[-6:], new_hash[-6:]),
-                    style="bold yellow",
-                )
-                do_lock(
-                    project,
-                    system=system,
-                    pre=pre,
-                    write=True,
-                    pypi_mirror=pypi_mirror,
-                    categories=categories,
-                )
-    # Write out the lockfile if it doesn't exist.
-    if not project.lockfile_exists and not skip_lock:
-        # Unless we're in a virtualenv not managed by pipenv, abort if we're
-        # using the system's python.
-        if (system or allow_global) and not (project.s.PIPENV_VIRTUALENV):
-            raise exceptions.PipenvOptionsError(
-                "--system",
-                "--system is intended to be used for Pipfile installation, "
-                "not installation of specific packages. Aborting.\n"
-                "See also: --deploy flag.",
-            )
-        else:
-            err.print(
-                "Pipfile.lock not found, creating...",
-                style="bold",
-            )
-            do_lock(
-                project,
-                system=system,
-                pre=pre,
-                write=True,
-                pypi_mirror=pypi_mirror,
-                categories=categories,
-            )
-    do_install_dependencies(
+
+    handle_lockfile(
         project,
-        dev=dev,
-        dev_only=dev_only,
-        allow_global=allow_global,
-        requirements_dir=requirements_dir,
-        pypi_mirror=pypi_mirror,
-        extra_pip_args=extra_pip_args,
-        categories=categories,
+        packages,
+        ignore_pipfile=ignore_pipfile,
         skip_lock=skip_lock,
+        system=system,
+        allow_global=allow_global,
+        deploy=deploy,
+        pre=pre,
+        pypi_mirror=pypi_mirror,
+        categories=categories,
     )
 
-    # Hint the user what to do to activate the virtualenv.
     if not allow_global and not deploy and "PIPENV_ACTIVE" not in os.environ:
         console.print(
             "To activate this project's virtualenv, run [yellow]pipenv shell[/yellow].\n"
